;(function($, window, document, undefined) {
  var today = new Date();


  // defaults definition
  var pluginName = "materialCalendar",
      defaults = {
        currentDate: today,
        selectedDate: today,

        month: {
          short: {
                   he: ["ינו'", "פבר'", "מרץ", "אפר'", "מאי", "יוני", "יולי", "אוג'", "ספט'", "אוק'", "נוב'", "דצמ'"],
                   en: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
                 },
          long:  {
                   he: ["ינואר", "פברואר", "מרץ", "אפריל", "מאי", "יוני", "יולי", "אוגוסט", "ספטמבר", 
                        "אוקטובר", "נובמבר", "דצמבר"],
                   en: ["Januar", "Februar", "March", "April", "May", "June", "July", "August", "September", 
                        "October", "November", "December"]
                 }
        },

        day: {
          short: {
                   he: ["א", "ב", "ג", "ד", "ה", "ו", "ש"],
                   en: ["S", "M", "T", "W", "T", "F", "S"]
                 },
          long: {
                  he: ["ראשון", "שני", "שלישי", "רביעי", "חמישי", "שישי", "שבת"],
                  en: ["Sunday", "Monday", "Thusday", "Wedsday", "Thursday", "Friday", "Saturday"]
                }
        },

        buttons: 
        {
          text: {
                  he: ["אישור", "ביטול"],
                  en: ["OK", "CANCEL"]
                },
          action: [
                    function ok(event){
                      console.log('ok', event.data.selectedDate);
                      
                      event.data.element.val(event.data.selectedDate);
                      event.data.element.parent().find('.modalBackground, .container').detach();

                    },
                    function cancel(event){
                      console.log('cancel');

                      event.data.element.parent().find('.modalBackground, .container').detach();
                    }
                  ]
        },
        navBtn: {
          left:   "<i class='material-icons'>keyboard_arrow_left</i>",
          right:  "<i class='material-icons'>keyboard_arrow_right</i>"
        },

        language: "en",

        setOnSelect: true  // TODO

      }
  
  // constructor
  function Plugin(element, options) {

    this.element = $(element);
    // console.log(element.value);

    // check for options existens
    if(options == undefined){ options = {} };
    if(element.value.length>0) {options.selectedDate = new Date(element.value)};
    
    this.options =        $.extend({}, defaults, options);

    this._dateSelected= {
        year:             this.options.selectedDate.getFullYear(),
        month:            this.options.selectedDate.getMonth(),
        day:              this.options.selectedDate.getDate(),
        weekDay:          this.options.selectedDate.getDay()
      };

    this._dateCurrent= {
      year:               this.options.selectedDate.getFullYear(),
      month:              this.options.selectedDate.getMonth(),
      day:                this.options.selectedDate.getDate(),
      weekDay:            this.options.selectedDate.getDay()
    };

    this._defaults=       this.options;
    this._name=           pluginName;

    this.element.on('focus',{this: this} ,function(event){
      event.data.this.init();
    });
  }

  function animateNewMonth(element, dir) {
   
    var elContainer = element.parent(),
        elWidth = elContainer.width()/3 * dir;

    element.animate({ "left": elWidth }, 120, "easeOutQuint", function animateMonth() {

      var elCurrent = element.eq(1 + dir);

      element.eq(1 + dir).detach();
      element.css("left", 0);

      switch (true) {
        case (dir > 0):

          elContainer.prepend(elCurrent);
          break;

        case (dir < 0):

          elContainer.append(elCurrent);
          break;

      };
    });
  };

  Plugin.prototype = {
    init: function init() {
      // Initialization logic here

      this._month = this._defaults.month;
      this._day = this._defaults.day;

      //set modal background
      this.element.before($("<div/>").addClass('modalBackground'));
      
      //start template building
      this.template = $("<div/>").addClass("noselect");

      switch(this._defaults.language){
        case "he": 
          this.template.css("direction","rtl");
          break;
        dafault: 
          this.template.css("direction","ltr");
          break;
      }

      // append calendar parts to the container
      this.render_head().appendTo(this.template);
      this.render_body().appendTo(this.template);

      this.render_footer().appendTo(this.template);
      
      // Fix float for ltr/rtl
      this.floatFix();

      // append container to the DOM
      this.element.after(this.template.addClass("container"));

      this.template.on("click",".month_day", {this: this} ,this.renderSelectedDate);
      this.ink();
      
    },
    
    render_head: function render_head() {
      var $template = $("<div/>").addClass("cal_side");

      //day box
      $("<div/>").addClass("day").text(this._day.long[this._defaults.language][this._dateSelected.weekDay]).appendTo($template);
      $("<div/>").addClass("month").text(this._month.short[this._defaults.language][this._dateSelected.month]).appendTo($template);
      $("<div/>").addClass("date").text(this._dateSelected.day).appendTo($template);
      $("<div/>").addClass("year").text(this._dateSelected.year).appendTo($template);

      return $template;
    },

    render_body: function render_body() {

      var $template = $("<div/>");

      // add head to month container
      this.render_month_head().appendTo($template);

      // add body to month container
      var className="",
          current = this._dateCurrent,
          month = this._month
          t = this,
          dir = 0;
          
      
      this.render_month_body().appendTo($template);

      return $template.addClass("cal_body");

    },

    render_month_view: function render_month_view() {

      var currDay =         this.current.day,
          currMonth =       this.current.month + this.pos,
          currYear =        this.current.year,

          t =               new Date(),
          today =           {
                              day:    t.getDate(),
                              month:  t.getMonth(),
                              year:   t.getFullYear()
                            },
          
          firstDayOfMonth = new Date(currYear, currMonth, 1).getDay() + 1,
          lastDayOfMonth =  new Date(currYear, currMonth + 1, 0).getDate(),
          currDayOfWeek =   new Date(currYear, currMonth, currDay).getDay(),

          $day,
          $template = $("<div/>").addClass("month_container");

      if (currMonth < 0) {
        currMonth = 11;
        currYear -= 1;
      } else if (currMonth > 11) {
        currMonth = 0;
        currYear += 1;
      };
      
      // Week days head
      for (var i = 0; i <= 6; i++) {
        $("<div/>").addClass("month_head").html(this.day[i]).appendTo($template);
      };

      // Empty place holders for last month
      for (var i = 1; i < firstDayOfMonth; i++) {
        $("<div/>").addClass("month_placeholder").html("").appendTo($template);
      };

      // Month days
      for (var i = 1; i <= lastDayOfMonth; i++) {
        $day = $("<div/>").addClass("month_day").text(i);

        // Color the current date if within the current month
        if (i === today.day && currMonth === today.month && currYear === today.year) 
          { $day.addClass("active"); }

        // Color the selected date if within the current month
        if (i == this.selected.day && currMonth === this.selected.month && currYear === this.selected.year) 
          { $day.addClass("selected"); }

        $template.append($day);

      };

      return $template;
    },

    renderSelectedDate: function renderSelectedDate(event){
      $(".month_day").removeClass("selected");
      $(this).addClass("selected");

      //set the new selected date
      event.data.this._dateSelected= {
        day:    parseInt($(this).text()),
        month:  event.data.this._dateCurrent.month,
        year:   event.data.this._dateCurrent.year,
        weekDay:event.data.this._dateCurrent.weekDay
      }

      //update selected date in cal_side (calendar header)
      event.data.this.template.find(".cal_side").detach();
      event.data.this.template.prepend(event.data.this.render_head.call(event.data.this));
      // Fix float for ltr/rtl
      event.data.this.floatFix();
      return false;
    },

    // Fix float for ltr/rtl
    floatFix: function floatFix(){
      switch(this._defaults.language){
        case "he":
          this.template.find("li").css("float", "left");
          this.template.find(".month_day, .month_placeholder, .month_head, .cal_action, >div").css("float","right");
          //this.template.find(".left").css("float","left");
          //this.template.find(".right").css("float","right");
          break;
          
        default:
          this.template.find("li").css("float", "right");
          this.template.find(".month_day, .month_placeholder, .month_head, .cal_action, >div").css("float","left");
          //this.template.find(".right").css("float","left");
          //this.template.find(".left").css("float","right");
          break;
          
      }
    },
    
    handler: function handler(event) {
      var element = event.data.this.template.find(".cal_container").children();

      var dir=0,
          nameLength = event.data.name.length;
      switch (event.data.name.substring(0,nameLength-4)) {
        case "right":
          dir = -1;
          break;
        case "left":
          dir = 1;
          break;
      }

      switch (event.data.name.substring(nameLength-3,nameLength)){
        case "RTL":
          event.data.current.month += dir;
          break;
        default:
          event.data.current.month -= dir;
          break;
      }

      if (event.data.current.month < 0) {
        event.data.current.month = 11;
        event.data.current.year -= 1;
      } else if (event.data.current.month > 11) {
        event.data.current.month = 0;
        event.data.current.year += 1;
      };

      animateNewMonth(element, dir);
      $(".cal_head>span").text(event.data.month.long[event.data.this._defaults.language][event.data.current.month] + " " + event.data.current.year);

      // Fix float for ltr/rtl
      event.data.this.floatFix();

      var defaults = {
        selected: event.data.this._dateSelected, 
        current: event.data.this._dateCurrent,
        day: event.data.this._day.short[event.data.this._defaults.language],
        pos: dir 
      }

      //replace the new month element with the old month element
      var newEl = event.data.this.render_month_view.call(defaults).html();
      element.eq(1+dir).html(newEl);

      return false;
    },

    navigationBtn: function navigationBtn(className, text) {
      var current = this._dateCurrent,
          month = this._month;

      // create element
      var $temp = $("<div/>").addClass(className)
      // add element content
      $temp.html("<span><strong>" + text + "</strong></span>")

      // add event handler
      $temp.on("click", { name: className , current: current, month: month, this:this}, this.handler);

      return $temp;
    },

    render_month_head: function render_month_head() {

      var $template = $("<div/>").addClass("cal_head"),
          $temp = $("<div/>"),
          langDir = "LTR";

      if(this._defaults.lang==="he"){langDir="RTL"}
      
      $template.append(this.navigationBtn("left "+langDir, this._defaults.navBtn.left));
      $template.append($("<span/>").html(this._month.long[this._defaults.language][this._dateCurrent.month] + " " + this._dateCurrent.year));
      $template.append(this.navigationBtn("right "+langDir, this._defaults.navBtn.right));

      return $template;
    },

    render_month_body: function render_month_body() {
      var defaults = {
        selected: this._dateSelected, 
        current: this._dateCurrent,
        day: this._day.short[this._defaults.language]
      }

      var $template = $("<div/>").addClass("cal_container");

      this.render_month_view.call($.extend(defaults, {pos: 1})).addClass("first").appendTo($template);
      
      this.render_month_view.call($.extend(defaults, {pos: 0})).addClass("second").appendTo($template);

      this.render_month_view.call($.extend(defaults, {pos:-1})).addClass("third").appendTo($template);

      return $template;
    },

    getSelectedDate: function getSelectedDate(){
      
      var selectedYear = this._dateSelected.year,
          selectedMonth = this._dateSelected.month+1,
          selectedDay = this._dateSelected.day;
      
      if(this._dateSelected.month<=9){selectedMonth = '0' + this._dateSelected.month;};
      if(this._dateSelected.day<=9){selectedDay = '0' + this._dateSelected.day;};
      return selectedYear + "-" + selectedMonth + "-" + selectedDay;
      
    },
    
    render_footer: function render_footer() {
      var $template = $("<div/>").addClass("cal_action"),
          lang = this._defaults.language,
          buttons = this._defaults.buttons.text[lang],
          actions = this._defaults.buttons.action;
      

      //build action container
      $("<ul/>").addClass("btn").appendTo($template);
      
      //add action buttons
      for (i=0; i<buttons.length; i++){
        $("<li/>").html("<a>" + buttons[i] + "</a>")
          .on('click',{
            element: this.element, 
            selectedDate: this.getSelectedDate.call(this),
          }, actions[i])
          .appendTo($template.find("ul"));
      }

      return $template;
    },
    
    ink: function() {
      var parent, ink, d, x, y;
      this.template.on("mousedown",".btn>",function(e){
        parent = $(this);
        //create .ink element if it doesn"t exist
        if(parent.find(".ink").length == 0)
        parent.prepend("<span class='ink'></span>");

        ink = parent.find(".ink");
        //incase of quick double clicks stop the previous animation
        ink.removeClass("animate");

        //set size of .ink
        if(!ink.height() && !ink.width())
        {
          //use parent"s width or height whichever is larger for the diameter to make a circle which can cover the entire element.
          d = Math.max(parent.outerWidth(), parent.outerHeight());
          ink.css({height: d, width: d});
        }

        //get click coordinates
        //logic = click coordinates relative to page - parent's position relative to page - half of self height/width to make it controllable from the center;
        x = e.pageX - parent.offset().left - ink.width()/2;
        y = e.pageY - parent.offset().top - ink.height()/2;

        //set the position and add class .animate
        ink.css({top: y+"px", left: x+"px"}).addClass("animate");
      });
    }
  };

  // A really lightweight plugin wrapper around the constructor, 
  // preventing against multiple instantiations
  $.fn[pluginName] = function(options) {
    return this.each(function() {
      if (!$.data(this, "plugin_" + pluginName)) {
        $.data(this, "plugin_" + pluginName,
          new Plugin(this, options));
      }
    });
  }

})(jQuery, window, document);

$(".cal").materialCalendar();